package com.androdome.commander;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.RegisteredEvent;
import com.cliffblaster.game.api.event.BlockChangeEvent;
import com.cliffblaster.game.api.event.BlockEventListener;
import com.cliffblaster.game.api.event.Event;
import com.cliffblaster.game.api.event.EventListener;
import com.cliffblaster.game.level.tile.Tile;
import com.cliffblaster.game.net.ServerHandler;
import com.cliffblaster.game.net.packet.Packet;
import com.cliffblaster.game.net.packet.Packet9Chat;

public class CuboidCommand extends Command
{
	MoreCommands mcBase;

	public CuboidCommand(MoreCommands base) {
		mcBase = base;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		return mcBase.perms.hasPermission(caller, "morecommands.building.cuboid");
	}

	@Override
	public boolean execute(final String[] arguments, final ServerHandler handler) {
		final int maxCuboid;
		String perm = mcBase.perms.getFirstMatchingPermission(handler, "morecommands.building.cuboid.size.");
		if (handler == null)
		{
			sendMessage(handler, "You cannot use this command from the console!", null);
			return true;
		}
		if (perm != null)
		{
			try
			{
				maxCuboid = Integer.parseInt(perm.replace("morecommands.building.cuboid.size.", ""));
			}
			catch (NumberFormatException e)
			{

				sendMessage(handler, "The permission could not be parsed!", ChatColor.RED);
				return true;
			}
		}
		else maxCuboid = Integer.MAX_VALUE;
		if (mcBase.eventHolder.get(handler) != null)
		{
			mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
			mcBase.eventHolder.remove(handler);
			handler.sendPacket((Packet) new Packet9Chat("&aCancelled previous event"));
			return true;
		}
		final int baseId;
		if (arguments.length > 1)
		{
			try
			{
				baseId = Integer.parseInt(arguments[1]);
				if (baseId >= Tile.tiles.length || (baseId != 0 && Tile.tiles[baseId] == null))
				{
					throw new NumberFormatException();
				}
			}
			catch (NumberFormatException ex)
			{
				sendMessage(handler, "That is not a valid block ID!", ChatColor.BRIGHT_RED);
				return true;
			}
		}
		else baseId = -1;
		handler.sendPacket((Packet) new Packet9Chat("&ePlace two blocks"));
		mcBase.eventHolder.put(handler, mcBase.registerEvent(Event.Type.BLOCK_CHANGE, (EventListener) new BlockEventListener() {
			int pos1x = -1;
			int pos1y = -1;
			int pos1z = -1;
			int id = -1;

			public void onBlockChange(final BlockChangeEvent event) {
				if (handler == event.getHandler() && !event.isCancelled())
				{
					event.cancel();
					if (this.pos1x <= -1)
					{
						this.id = event.getNewId();
						if (baseId >= 0)
						{
							this.id = baseId;
							handler.sendPacket((Packet) new Packet9Chat("&ePlaced position 1, using ID = " + this.id));
						}
						else
						{
							handler.sendPacket((Packet) new Packet9Chat("&ePlaced position 1"));
						}
						handler.getLevel().setTile(event.getX(), event.getY(), event.getZ(), event.getOldId(), false);
						this.pos1x = event.getX();
						this.pos1y = event.getY();
						this.pos1z = event.getZ();
						if (!handler.getAllowedBlocks().contains(this.id))
						{
							handler.sendPacket((Packet) new Packet9Chat("&cYou cannot place blocks with ID = " + this.id));
							mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
							mcBase.eventHolder.remove(handler);
						}
					}
					else
					{
						mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
						mcBase.eventHolder.remove(handler);
						int willChange = 0;
						for (int x = Math.min(this.pos1x, event.getX()); x <= Math.max(this.pos1x, event.getX()); ++x)
						{
							for (int y = Math.min(this.pos1y, event.getY()); y <= Math.max(this.pos1y, event.getY()); ++y)
							{
								for (int z = Math.min(this.pos1z, event.getZ()); z <= Math.max(this.pos1z, event.getZ()); ++z)
								{
									if (handler.getLevel().getTile(x, y, z) != this.id)
									{
										willChange++;
									}
								}
							}
						}
						if (willChange > maxCuboid)
						{
							handler.sendPacket(new Packet9Chat("&cCuboid too large: " + willChange));
							handler.sendPacket(new Packet9Chat("&cYour maximum: " + maxCuboid));
							return;
						}
						ByteArrayOutputStream baos = null;
						DataOutputStream gzip = null;
						try
						{
							baos = new ByteArrayOutputStream();
							gzip = new DataOutputStream(new GZIPOutputStream(baos));
							for (int x = Math.min(this.pos1x, event.getX()); x <= Math.max(this.pos1x, event.getX()); ++x)
							{
								for (int y = Math.min(this.pos1y, event.getY()); y <= Math.max(this.pos1y, event.getY()); ++y)
								{
									for (int z = Math.min(this.pos1z, event.getZ()); z <= Math.max(this.pos1z, event.getZ()); ++z)
									{
										if (handler.getLevel().getTile(x, y, z) != this.id)
										{
											gzip.writeShort(x);
											gzip.writeShort(y);
											gzip.writeShort(z);
											gzip.writeShort(handler.getLevel().getTile(x, y, z));
											handler.getLevel().setTile(x, y, z, this.id, false);
											
										}
									}
								}
							}
							gzip.close();
							gzip = null;
							baos.close();
							mcBase.undoBuffer.put(handler, baos.toByteArray());
							baos = null;
							handler.sendPacket((Packet) new Packet9Chat("&aFinished cuboid! Changed " + willChange + " blocks."));
						}
						catch (Exception ex)
						{
							handler.sendPacket((Packet) new Packet9Chat("&4An unknown error has occured on the server"));
							if(gzip != null) 
								try
								{
									gzip.close();
								}
								catch (IOException e)
								{
									e.printStackTrace();
								}
							if(baos != null) 
								try
								{
									baos.close();
								}
								catch (IOException e)
								{
									e.printStackTrace();
								}
							ex.printStackTrace();
						}
					}
				}
			}
		}, Event.Priority.NORMAL));
		return true;
	}

	@Override
	public String getHelpDesc() {
		return "Makes a cuboid with the selected block";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " [id]", ChatColor.CYAN);
	}
}
