package com.androdome.commander;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.RegisteredEvent;
import com.cliffblaster.game.api.event.BlockChangeEvent;
import com.cliffblaster.game.api.event.BlockEventListener;
import com.cliffblaster.game.api.event.Event;
import com.cliffblaster.game.net.ServerHandler;
import com.cliffblaster.game.net.packet.Packet;
import com.cliffblaster.game.net.packet.Packet9Chat;

public class ReplaceCommand extends Command
{
	MoreCommands mcBase;

	public ReplaceCommand(MoreCommands base) {
		mcBase = base;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		return mcBase.perms.hasPermission(caller, "morecommands.building.replace");
	}

	@Override
	public boolean execute(final String[] arguments, final ServerHandler handler) {
		final int maxCuboid;
		String perm = mcBase.perms.getFirstMatchingPermission(handler, "morecommands.building.replace.size.");
		if (handler == null)
		{
			sendMessage(handler, "You cannot use this command from the console!", null);
			return true;
		}
		if (perm != null)
		{
			try
			{
				maxCuboid = Integer.parseInt(perm.replace("morecommands.building.replace.size.", ""));
			}
			catch (NumberFormatException e)
			{

				sendMessage(handler, "The permission could not be parsed!", ChatColor.RED);
				return true;
			}
		}
		else maxCuboid = Integer.MAX_VALUE;
		if (mcBase.eventHolder.get(handler) != null)
		{
			mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
			mcBase.eventHolder.remove(handler);
			handler.sendPacket((Packet) new Packet9Chat("&aCancelled previous event"));
			return true;
		}
		handler.sendPacket((Packet) new Packet9Chat("&ePlace two blocks"));
		if (arguments.length > 2)
		{
			try
			{
				final int id = Integer.parseInt(arguments[1].trim());
				final int newId = Integer.parseInt(arguments[2].trim());
				mcBase.eventHolder.put(handler, mcBase.registerEvent(Event.Type.BLOCK_CHANGE, new BlockEventListener() {
					int pos1x = -1;
					int pos1y = -1;
					int pos1z = -1;

					public void onBlockChange(final BlockChangeEvent event) {
						if (handler == event.getHandler() && !event.isCancelled())
						{
							event.cancel();
							if (this.pos1x <= -1)
							{
								handler.sendPacket((Packet) new Packet9Chat("&ePlaced position 1"));
								handler.getLevel().setTile(event.getX(), event.getY(), event.getZ(), event.getOldId(), false);
								this.pos1x = event.getX();
								this.pos1y = event.getY();
								this.pos1z = event.getZ();
								if (!handler.getAllowedBlocks().contains(id) || !handler.getAllowedBlocks().contains(newId))
								{
									handler.sendPacket((Packet) new Packet9Chat("&cYou cannot place blocks with ID = " + id));
									mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
									mcBase.eventHolder.remove(handler);
								}
							}
							else
							{
								mcBase.getManager().unregisterEvent((RegisteredEvent) mcBase.eventHolder.get(handler));
								mcBase.eventHolder.remove(handler);
								int willChange = 0;
								for (int x = Math.min(this.pos1x, event.getX()); x <= Math.max(this.pos1x, event.getX()); ++x)
								{
									for (int y = Math.min(this.pos1y, event.getY()); y <= Math.max(this.pos1y, event.getY()); ++y)
									{
										for (int z = Math.min(this.pos1z, event.getZ()); z <= Math.max(this.pos1z, event.getZ()); ++z)
										{
											if (handler.getLevel().getTile(x, y, z) == id)
											{
												willChange++;
											}
										}
									}
								}
								if (willChange > maxCuboid)
								{
									handler.sendPacket(new Packet9Chat("&Replacement area too large: " + willChange));
									handler.sendPacket(new Packet9Chat("&cYour maximum: " + maxCuboid));
									return;
								}
								ByteArrayOutputStream baos = null;
								DataOutputStream gzip = null;
								try
								{
									baos = new ByteArrayOutputStream();
									gzip = new DataOutputStream(new GZIPOutputStream(baos));
									for (int x = Math.min(this.pos1x, event.getX()); x <= Math.max(this.pos1x, event.getX()); ++x)
									{
										for (int y = Math.min(this.pos1y, event.getY()); y <= Math.max(this.pos1y, event.getY()); ++y)
										{
											for (int z = Math.min(this.pos1z, event.getZ()); z <= Math.max(this.pos1z, event.getZ()); ++z)
											{
												if (handler.getLevel().getTile(x, y, z) == id)
												{
													gzip.writeShort(x);
													gzip.writeShort(y);
													gzip.writeShort(z);
													gzip.writeShort(handler.getLevel().getTile(x, y, z));
													handler.getLevel().setTile(x, y, z, newId, false);

												}
											}
										}
									}
									gzip.close();
									gzip = null;
									baos.close();
									mcBase.undoBuffer.put(handler, baos.toByteArray());
									baos = null;
									handler.sendPacket(new Packet9Chat("&aFinished replacement! Changed " + willChange + " blocks."));
								}
								catch (Exception ex)
								{
									handler.sendPacket(new Packet9Chat("&4An unknown error has occured on the server"));
									if (gzip != null) try
									{
										gzip.close();
									}
									catch (IOException e)
									{
										e.printStackTrace();
									}
									if (baos != null) try
									{
										baos.close();
									}
									catch (IOException e)
									{
										e.printStackTrace();
									}
									ex.printStackTrace();
								}
							}
						}
					}
				}, Event.Priority.NORMAL));
			}
			catch (NumberFormatException ex)
			{
				handler.sendPacket(new Packet9Chat("&cThat is not a valid number"));
			}
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Replaces one block with another in the selected area";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <block id to replace> <block id>", ChatColor.CYAN);
	}

}
