package com.androdome.commander;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.abstraction.Server;
import com.cliffblaster.game.net.ServerHandler;

public class UndoCommand extends Command
{
	MoreCommands mcBase;

	public UndoCommand(MoreCommands base) {
		mcBase = base;
	}

	@Override
	public boolean hasPermission(String command[], ServerHandler caller) {
		if(command.length > 1)
			return mcBase.perms.hasPermission(caller, "morecommands.building.undo.other");
		return mcBase.perms.hasPermission(caller, "morecommands.building.undo.self");
	}

	@Override
	public boolean execute(String[] arguments, ServerHandler caller) {
		if (arguments.length < 2 && caller == null)
		{
			sendMessage(caller, "You cannot use this command from the console!", null);
			return true;
		}
		ServerHandler thePlayer = caller;
		if(arguments.length > 1)
			thePlayer = Server.getHandle(arguments[1]);
		if(thePlayer != null)
		{
			ByteArrayInputStream bais = null;
			DataInputStream dis = null;
			try
			{
				if(mcBase.undoBuffer.containsKey(thePlayer))
				{
					bais = new ByteArrayInputStream(mcBase.undoBuffer.remove(thePlayer));
					dis = new DataInputStream(new GZIPInputStream(bais));
					try{
						while(true)
							caller.getLevel().setTile(dis.readShort(),dis.readShort(),dis.readShort(),dis.readShort(),true);
					}catch(java.io.EOFException ex)
					{}
					bais.close();
					dis.close();
					sendMessage(caller, "Undo complete!", ChatColor.LIME);
				}
				else sendMessage(caller, "Nothing left to undo!", ChatColor.LIME);
			}
			catch (Exception ex)
			{

				sendMessage(caller, "An unknown error has occured on the server", ChatColor.RED);
				if(bais != null) 
					try
					{
						bais.close();
					}
					catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				if(dis != null) 
					try
					{
						dis.close();
					}
					catch (IOException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				ex.printStackTrace();
			}
		}
		else
			sendMessage(caller, arguments[1] + " doesn't exist or is ambiguous", ChatColor.RED);
		return true;
	}

	@Override
	public String getHelpDesc() {
		return "Undoes your or another player's command";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " [player]", ChatColor.CYAN);
	}

}
