package com.androdome.perms;

import org.json.*;

import com.cliffblaster.game.api.JavaPlugin;
import com.cliffblaster.game.net.ServerHandler;
import com.cliffblaster.game.net.packet.Packet9Chat;

import java.io.*;
import java.util.*;


class PermissionsManager
{
    JavaPlugin plugin;
    File jsonPerms;
    JSONObject json;
    
    public PermissionsManager(final JavaPlugin permissions) {
        this.plugin = permissions;
        this.jsonPerms = new File(plugin.getPluginDirectory(), "perms.json");
        this.loadPerms();
    }
    
    void loadPerms() {
        if (!this.jsonPerms.exists()) {
            try {
                this.jsonPerms.createNewFile();
                this.json = new JSONObject();
                final JSONObject groupObject = new JSONObject();
                groupObject.put("*", new JSONArray().put("social.*").put("server.public.*").put("polyterra.standard.*"));
                this.json.put("group", groupObject);
                final JSONObject player = new JSONObject();
                this.json.put("player", player);
                final BufferedWriter writer = new BufferedWriter(new FileWriter(this.jsonPerms));
                writer.write(this.json.toString(1));
                writer.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                this.json = new JSONObject(new Scanner(this.jsonPerms).useDelimiter("\\Z").next());
            }
            catch (JSONException e2) {
                this.json = null;
                e2.printStackTrace();
            }
            catch (FileNotFoundException e3) {
                this.json = null;
                e3.printStackTrace();
            }
        }
    }
    
    boolean hasPermissions(String permission, final ServerHandler playerHandler) {
    	permission = permission.trim().toLowerCase();
        if (playerHandler == null || playerHandler.isAdmin) {
            return true;
        }
        final LinkedHashSet<String> permlist = this.getPermList(permission.toLowerCase(), playerHandler);
        return permlist != null && !permlist.contains("-" + permission) && (permlist.contains("*") || permlist.contains(permission) || checkSteppingAsterisk(permission, permlist));
    }
    
    String getFirstMatchingPermission(final String permission, final ServerHandler playerHandler) {
        final LinkedHashSet<String> permlist = this.getPermList(permission.toLowerCase(), playerHandler);
        for(String i : permlist)
        {
        	if(i.startsWith(permission))
        	{
        		return i;
        	}
        }
        return null;
    }
    
    private static boolean checkSteppingAsterisk(final String permission, final LinkedHashSet<String> list) {
        final String[] permBreakdown = permission.split("\\.");
        for (int i = 1; i <= permBreakdown.length; ++i) {
            String perm = "";
            for (int x = 0; x < i - 1; ++x) {
                perm = String.valueOf(perm) + permBreakdown[x] + ".";
            }
            perm = String.valueOf(perm) + "*";
            if (list.contains(perm)) {
                return true;
            }
        }
        return false;
    }
    
    LinkedHashSet<String> getPermList(final String permission, final ServerHandler player) {
        final LinkedHashSet<String> perms = new LinkedHashSet<String>();
        final String playerName = player.getPlayerName().toLowerCase();
        try {
            if (this.json == null) {
                throw new Exception("JSON not valid");
            }
            final JSONObject groups = this.json.getJSONObject("group");
            final JSONArray defaultGroup = groups.getJSONArray("*");
            for (int i = 0; i < defaultGroup.length(); ++i) {
                perms.add(defaultGroup.getString(i));
            }
            final JSONObject players = this.json.getJSONObject("player");
            if (players.has(playerName)) {
                final JSONObject playerJson = players.getJSONObject(playerName);
                JSONArray playerGroups = null;
                if (playerJson.has("groups")) {
                    playerGroups = playerJson.getJSONArray("groups");
                }
                JSONArray playerPerms = null;
                if (playerJson.has("perms")) {
                    playerPerms = playerJson.getJSONArray("perms");
                }
                if (playerPerms != null) {
                    for (int j = 0; j < playerPerms.length(); ++j) {
                        final String str = playerPerms.getString(j);
                        if (str.startsWith("+")) {
                            perms.remove("-" + str.replaceFirst("+", ""));
                        }
                        else {
                            perms.add(str);
                        }
                    }
                }
                if (playerGroups != null) {
                    for (int x = 0; x < playerGroups.length(); ++x) {
                        if (groups.has(playerGroups.getString(x))) {
                            final JSONArray group = groups.getJSONArray(playerGroups.getString(x));
                            for (int k = 0; k < group.length(); ++k) {
                                final String str2 = group.getString(k);
                                if (str2.startsWith("+")) {
                                    perms.remove("-" + str2.replaceFirst("\\+", ""));
                                }
                                else {
                                    perms.add(str2);
                                }
                            }
                        }
                    }
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
            player.sendPacket(new Packet9Chat("&cPermissions are corrupt or not in JSON format,"));
            player.sendPacket(new Packet9Chat("&cPlease contact the server administrator."));
            return null;
        }
        return perms;
    }
}
