package com.androdome.polyterra;

import java.io.File;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.event.Event.Priority;
import com.cliffblaster.game.api.event.Event.Type;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.api.event.ServerCommandEvent;
import com.cliffblaster.game.api.event.ServerEventListener;
import com.cliffblaster.game.net.ServerHandler;


public class DeleteCommand extends Command
{

	final MainPlugin plugin;

	public DeleteCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event;
		if(command.length > 1)
			event = plugin.getManager().requestPermission(caller, "polyterra.admin.delete."+command[1].trim().toLowerCase());
		else event = plugin.getManager().requestPermission(caller, "polyterra.admin.delete");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return caller == null || caller.isAdmin;
	}

	@Override
	public boolean execute(String[] args, final ServerHandler caller) {
		if (args.length > 1)
		{
			final String levelToDestroy = args[1].toLowerCase();
			if (plugin.nameToLevel.containsKey(levelToDestroy))
			{
				sendMessage(caller, "You must unload the level before deletion", ChatColor.RED);
				return true;
			}
			if (new File(plugin.worldsDir, levelToDestroy + ".dat").exists())
			{
				sendMessage(caller, "Are you sure? Use the command /yes or /no", ChatColor.YELLOW);
				plugin.addEvent(caller, plugin.registerEvent(Type.SERVER_COMMAND, new ServerEventListener() {
					public void onServerCommand(ServerCommandEvent chatEvent) {
						chatEvent.cancel();
						if (chatEvent.getMessage().trim().equalsIgnoreCase("yes"))
						{
							sendMessage(caller, "Deleting level...", ChatColor.YELLOW);
							if (new File(plugin.worldsDir, levelToDestroy + ".dat").delete())
							{
								new File(plugin.worldsDir, levelToDestroy + ".properties").delete();
								sendMessage(caller, "Deleted!", ChatColor.LIME);
							}
							else sendMessage(caller, "Failed to delete! (Was it already removed?)", ChatColor.RED);
						}
						else sendMessage(caller, "Cancelled!", ChatColor.LIME);
						plugin.clearEvents(caller);
					}
				}, Priority.REALTIME));
			}
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Deletes a level that exists but is not currently loaded";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name>", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}