package com.androdome.polyterra;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.net.ServerHandler;


public class GetPropertiesCommand extends Command
{

	final MainPlugin plugin;

	public GetPropertiesCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event;
		if(command.length > 1)
			event = plugin.getManager().requestPermission(caller, "polyterra.admin.getproperties."+command[1].trim().toLowerCase());
		else event = plugin.getManager().requestPermission(caller, "polyterra.admin.getproperties");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return caller == null || caller.isAdmin;
	}


	@Override
	public boolean execute(String[] args, ServerHandler caller) {
		if(args.length > 1)
		{
			LevelData levelData;
			if(args[1].trim().toLowerCase().equalsIgnoreCase("?") && caller != null)
				levelData = plugin.getDataFromLevel(caller.getLevel());
			else levelData = plugin.nameToLevel.get(args[1].trim().toLowerCase());
			if(levelData == null)
			{
				sendMessage(caller, "World not loaded", ChatColor.RED);
			}
			else
			{
				sendMessage(caller, "Properties for " + levelData.levelName, ChatColor.BLUE);
				sendMessage(caller, "desc: " + levelData.levelDisplayName, ChatColor.YELLOW);
				sendMessage(caller, "motd: " + levelData.motd, ChatColor.YELLOW);
				sendMessage(caller, "autoload: " + levelData.autoload, ChatColor.YELLOW);
				sendMessage(caller, "tick: " + levelData.ticks, ChatColor.YELLOW);
				sendMessage(caller, "cancompress: " + levelData.levelCanCompress, ChatColor.YELLOW);
				sendMessage(caller, "iscompressed: " + levelData.levelIsCompressed, ChatColor.ORANGE);
				if(levelData.levelIsCompressed)
					sendMessage(caller, "compressedsize: " + (int)Math.ceil(levelData.level.blocks.length/1024F) + "KB", ChatColor.ORANGE);
				sendMessage(caller, "players: " + levelData.connectedPlayers.size(), ChatColor.ORANGE);
			}
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Gets the property of a level";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name/?>", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}