package com.androdome.polyterra;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.abstraction.Server;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.net.ServerHandler;
import com.cliffblaster.game.net.packet.Packet9Chat;

public class JoinCommand extends Command
{

	final MainPlugin plugin;

	public JoinCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event;
		if(command.length > 1)
			event = plugin.getManager().requestPermission(caller, "polyterra.standard.join."+command[1].trim().toLowerCase());
		else event = plugin.getManager().requestPermission(caller, "polyterra.standard.join");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return true;
	}


	@Override
	public boolean execute(String[] args, final ServerHandler caller) {
		if(caller == null)
		{
			sendMessage(caller, "You cannot join a level from the console!", ChatColor.RED);
			return true;
		}
		if(args.length > 1)
		{
			final String name = args[1].toLowerCase();
			final LevelData data = plugin.nameToLevel.get(name);
			final LevelData oldLevel = plugin.getDataFromLevel(caller.getLevel());
			if(data != null)
			{
				if(caller.getLevel().equals(data.level))
				{
					sendMessage(caller, "You are already in this level!", ChatColor.YELLOW);
					return true;
				}
				else if(oldLevel != null)
				{
					oldLevel.connectedPlayers.remove(caller);
				}
				sendMessage(caller, "Joining...", ChatColor.GREEN);
				new Thread(){
					public void run()
					{
						if(data.levelIsCompressed)
						{
							data.decompress();
							sendMessage(caller, "Uncompressing...", ChatColor.GREEN);
						}
						Server.broadcast(new Packet9Chat((caller.getFullPlayerName().startsWith("&") ? "" : ChatColor.PURPLE) + caller.getFullPlayerName()+" "+ChatColor.PURPLE+"went to another level!"), caller, caller.getLevel());
						caller.setLevel(data.level, "Joining " + data.levelDisplayName, data.motd);
						data.connectedPlayers.add(caller);
						Server.broadcast(new Packet9Chat((caller.getFullPlayerName().startsWith("&") ? "" : ChatColor.PURPLE) + caller.getFullPlayerName()+" "+ChatColor.PURPLE+"has joined the level!"), null, caller.getLevel());
						sendMessage(caller, "Joined!", ChatColor.GREEN);
						if(oldLevel != null && oldLevel.levelCanCompress && oldLevel.connectedPlayers.size() == 0)
						{
							try
							{
								oldLevel.compress();
							}
							catch (InterruptedException e)
							{
								e.printStackTrace();
							}
						}
					}
				}.start();
			}
			else sendMessage(caller, "The level \"" + name + "\" is not loaded", ChatColor.RED);
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Joins a loaded level";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name>", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}