package com.androdome.polyterra;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.net.ServerHandler;

public class ListCommand extends Command
{

	public int MAX_PAGE_SIZE = 8;
	
	public void printListPage(int page, ServerHandler caller) {
		ArrayList<String> levels = getAllAllowedLevels(caller);
		int pages = (int) Math.floor((double) levels.size() / (double) MAX_PAGE_SIZE);
		if (page > pages)
			sendMessage(caller, "You have reached the last page of the help command", ChatColor.YELLOW);
		else {
			sendMessage(caller, "--------------Levels page " + (page + 1) + " out of " + (pages + 1) + "--------------", ChatColor.NONE);
			for (int i = page * MAX_PAGE_SIZE; i < Math.min(levels.size(), page * MAX_PAGE_SIZE + 5); i++) {
				sendMessage(caller, levels.get(i), ChatColor.MAGENTA);
			}
		}
	}
	
	private ArrayList<String> getAllAllowedLevels(ServerHandler caller) {
		ArrayList<String> levels = new ArrayList<String>();
		ArrayList<String> levelsNotLoaded = new ArrayList<String>();
		for(File f : this.plugin.worldsDir.listFiles())
		{
			if(!f.getName().endsWith(".dat"))
				continue;
			String name = f.getName().replace(".dat", "").toLowerCase();
			PermissionRequestEvent event = plugin.getManager().requestPermission(caller, "polyterra.standard.join."+name);
			if(!event.wasHandled() || event.hasPermission())
			{
				if(plugin.nameToLevel.containsKey(name))
					levels.add(name + "&f: " + plugin.nameToLevel.get(name).levelDisplayName + " (" + plugin.nameToLevel.get(name).connectedPlayers.size() + " playing)");
				else
					levelsNotLoaded.add(name + ChatColor.GREY+": level not loaded");
			}

		}
		Collections.sort(levels);
		Collections.sort(levelsNotLoaded);
		levels.addAll(levelsNotLoaded);
		return levels;
	}

	final MainPlugin plugin;

	public ListCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event = requestPermission(caller, "polyterra.standard.list");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return true;
	}

	@Override
	public boolean execute(String[] arguments, ServerHandler caller) {
		if (arguments.length >= 2) {
			printListPage(Integer.parseInt(arguments[1]) - 1, caller);
		} else
			printListPage(0, caller);
		return true;
	}

	@Override
	public String getHelpDesc() {
		return "Lists all unloaded and levels";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " [page]", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}