package com.androdome.polyterra;

import java.io.File;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.event.Event.Priority;
import com.cliffblaster.game.api.event.Event.Type;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.api.event.ServerCommandEvent;
import com.cliffblaster.game.api.event.ServerEventListener;
import com.cliffblaster.game.net.ServerHandler;


public class RenameCommand extends Command
{

	final MainPlugin plugin;

	public RenameCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event;
		if(command.length > 1)
			event = requestPermission(caller, "polyterra.admin.rename."+command[1].trim().toLowerCase());
		else event = requestPermission(caller, "polyterra.admin.rename");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return caller == null || caller.isAdmin;
	}

	@Override
	public boolean execute(String[] args, final ServerHandler caller) {
		if (args.length > 2)
		{
			final String levelToRename = args[1].toLowerCase();
			final String toRenameTo = args[2].toLowerCase();
			try
			{
				if(new File(plugin.worldsDir, toRenameTo + ".dat").createNewFile())
				{
					new File(plugin.worldsDir, toRenameTo + ".dat").delete();
				}
				else
				{
					sendMessage(caller, "That is not a valid filename!", ChatColor.RED);
					return true;
				}
			}
			catch(Exception ex)
			{
				sendMessage(caller, "That is not a valid filename!", ChatColor.RED);
				return true;
			}

			final File newLevelFile = new File(plugin.worldsDir, toRenameTo + ".dat");
			final File newDataFile = new File(plugin.worldsDir, toRenameTo + ".properties");
			
			if (plugin.nameToLevel.containsKey(levelToRename))
			{
				sendMessage(caller, "You must unload the level before renaming it", ChatColor.RED);
				return true;
			}
			if (new File(plugin.worldsDir, levelToRename + ".dat").exists())
			{
				sendMessage(caller, "Renaming a level can cause other plugins", ChatColor.RED);
				sendMessage(caller, "    to not see the level, and may require", ChatColor.RED);
				sendMessage(caller, "    recreating plugin data files.", ChatColor.RED);
				sendMessage(caller, "Are you sure? Use the command /yes or /no", ChatColor.YELLOW);
				plugin.addEvent(caller, plugin.registerEvent(Type.SERVER_COMMAND, new ServerEventListener() {
					public void onServerCommand(ServerCommandEvent chatEvent) {
						chatEvent.cancel();
						if (chatEvent.getMessage().trim().equalsIgnoreCase("yes"))
						{
							sendMessage(caller, "Renaming level...", ChatColor.YELLOW);
							if (new File(plugin.worldsDir, levelToRename + ".dat").renameTo(newLevelFile))
							{
								new File(plugin.worldsDir, levelToRename + ".properties").renameTo(newDataFile);
								sendMessage(caller, "Renamed!", ChatColor.LIME);
							}
							else sendMessage(caller, "Failed to rename! (Does it exist?)", ChatColor.RED);
						}
						else sendMessage(caller, "Cancelled!", ChatColor.LIME);
						plugin.clearEvents(caller);
					}
				}, Priority.REALTIME));
			}
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Renames a level that exists but is not currently loaded";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name> <newname>", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}