package com.androdome.polyterra;

import com.cliffblaster.game.ChatColor;
import com.cliffblaster.game.Command;
import com.cliffblaster.game.api.event.PermissionRequestEvent;
import com.cliffblaster.game.net.ServerHandler;


public class SetPropertyCommand extends Command
{

	final MainPlugin plugin;

	public SetPropertyCommand(MainPlugin plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean hasPermission(String[] command, ServerHandler caller) {
		PermissionRequestEvent event;
		if(command.length > 1)
			event = requestPermission(caller, "polyterra.level.setproperty."+command[1].trim().toLowerCase());
		else
			event = requestPermission(caller, "polyterra.level.setproperty");
		if(event.wasHandled())
		{
			return event.hasPermission();
		}
		return caller == null || caller.isAdmin;
	}


	@Override
	public boolean execute(String[] args, ServerHandler caller) {
		if(args.length > 3)
		{
			LevelData levelData;
			if(args[1].trim().toLowerCase().equalsIgnoreCase("?") && caller != null)
				levelData = plugin.getDataFromLevel(caller.getLevel());
			else levelData = plugin.nameToLevel.get(args[1].trim().toLowerCase());
			String propertyName = args[2].trim();
			if(levelData == null)
			{
				sendMessage(caller, "World not loaded", ChatColor.RED);
			}
			else if(propertyName.equalsIgnoreCase("autoload")
					|| propertyName.equalsIgnoreCase("tick")
					|| propertyName.equalsIgnoreCase("cancompress")
					|| propertyName.equalsIgnoreCase("hypersprint")
					|| propertyName.equalsIgnoreCase("flying"))
			{
				try
				{
					Boolean value = Boolean.parseBoolean(args[3].trim());
					if(propertyName.equalsIgnoreCase("autoload")) levelData.autoload = value;
					else if(propertyName.equalsIgnoreCase("tick")) levelData.ticks = value;
					else if(propertyName.equalsIgnoreCase("cancompress"))
					{
						levelData.levelCanCompress = value;
						if(value && levelData.connectedPlayers.size() == 0)
						{
							levelData.compress();
						}
						else if(!value)
						{
							levelData.decompress();
						}
					}
					sendMessage(caller, "Property Set!", ChatColor.LIME);
				}catch(Exception ex)
				{
					sendMessage(caller, "Invalid value '"+"' for type " + propertyName.substring(0,1).toUpperCase() + propertyName.substring(1).toLowerCase(), ChatColor.RED);
				}
			}
			else if(propertyName.equalsIgnoreCase("motd") || propertyName.equalsIgnoreCase("desc"))
			{
				String value = "";
				for(int i = 3; i < args.length; i++)
					value += args[i] + " ";
				value.trim();
				if(propertyName.equalsIgnoreCase("motd")) levelData.motd = value;
				else if(propertyName.equalsIgnoreCase("desc")) levelData.levelDisplayName = value;
				sendMessage(caller, "Property Set!", ChatColor.LIME);
			}
			else
			{
				sendMessage(caller, "Invalid property " + propertyName.substring(0,1).toUpperCase() + propertyName.substring(1).toLowerCase(), ChatColor.RED);
				return false;
			}
			return true;
		}
		return false;
	}

	@Override
	public String getHelpDesc() {
		return "Sets the property of a level";
	}

	@Override
	public void printUsage(String[] arguments, ServerHandler caller) {
		sendMessage(caller, "/" + arguments[0] + " <name/?> <property name> <value>", ChatColor.CYAN);
		sendMessage(caller, "Properties: autoload, tick, motd, desc, hypersprint, flying, cancompress", ChatColor.CYAN);
	}

	void sendMessagePublic(ServerHandler caller, String message, ChatColor color) {
		this.sendMessage(caller, message, color);
	}
}